# encoding: utf-8

require 'net/pop'
require 'net/http'

T_HOST = 'twitter.com'
T_URI = '/statuses/update.xml'
I_HOST = 'is.gd'
I_BASE = 'api.php?longurl='

class PDMLTweet

  def initialize(g_u, g_p, t_u, t_p)
    @g_user = g_u
    @g_pass = g_p
    @twitter = Net::HTTP.new T_HOST
    @tweet = Net::HTTP::Post.new T_URI
    @tweet.basic_auth(t_u, t_p)
    @is_gd = Net::HTTP.new I_HOST
  end

  def shorten_link(long)
    uri = Net::HTTP::Get.new "#{I_BASE}#{long}"
    response = nil
    @is_gd.start { |connection| response = connection.request uri }

    if response.kind_of? Net::HTTPSuccess
      response.body
    else
      long
    end
  end

  def make_message(title, author, pre, link)
    
    # 'left' is the number bytes we can still stuff in the message;
    #   bytes, not characters!
    left = 140 - (link.bytesize + 1) # +1 for space in front of link

    if title.bytesize > left
      title = title[0 .. (left - 3)] # 3 for the "..."
      return "#{title}... #{link}"
    end

    left -= title.bytesize

    # add the author if there's room
    if author && (author.bytesize < (left - 6))
      title = "#{title}, by #{author}."
      left -= (author.bytesize + 6)
    end

    # done unless there's a reasonable amount of space
    return "#{title} #{link}" if left < 20

    # 'pre' is the text before the first hyperlink
    extra = " “” ".bytesize
    if pre.bytesize > (left - extra)
      pre = pre[0 .. (left - (extra + 3))]
      pre.gsub!(/\s+\S+$/, '') # don't break in the middle of a word
      pre += "..."
    end
    return "#{title} “#{pre}” #{link}"
  end

  def find_link(text)
    pre = link = nil
    if text =~ /^(.*?)((\s+(http:\/\/\S+))|(<(http:\/\/[^>]*)>))/m
      pre = $1
      link = $4 ? $4 : $6
    end
    [ pre, link ]
  end

  def tweet(title, author, text)

    pre, link = find_link(text)
    if link
      link = shorten_link link
      message = make_message(title, author, pre, link)
      message.gsub!('"', "'")

      @twitter.start do |connection|
        # if it doesn't work, oh well
        connection.request(@tweet, "status=#{message}\r\n\r\n")
      end
    end
  end

  def find_title(subjects)
    return nil unless subjects[0]
    if subjects[0] =~ /^Subject:\s+[PG]ESO\s*[-:]*\s*(\S.*)$/
      $1
    else
      nil
    end
  end
    
  def scan
    Net::POP3.enable_ssl(OpenSSL::SSL::VERIFY_NONE)
    Net::POP3.start('pop.gmail.com', 995, @g_user, @g_pass) do |pop|
      pop.each_mail do |mail|
        header = mail.header
        headers = header.split("\r\n")
        title = find_title(headers.grep(/^Subject:/))
        if title
          authors = headers.grep(/^From:/)
          if (authors[0])
            author = authors[0].gsub(/\s*<.*$/, '').gsub('"', '').gsub(/^From:\s*/, '')
          else
            author = nil
          end
          tweet(title, author, mail.pop[header.length .. -1])
          sleep 5 # Twitter doesn't like you pushing it too hard
        end
        mail.delete
      end
    end
  end
end
