$:.unshift File.join(File.dirname(__FILE__), "..", "src")

require 'test/unit'
require 'pdml-tweet.rb'

class TestPDMLTweet < Test::Unit::TestCase

  def setup
    @words = [ [], [], [] ]
    8.times do |i|
      @words[0] << 'a' * (i + 1) 
      @words[1] << 'b' * (i + 1) 
      @words[2] << 'c' * (i + 1) 
    end
    srand(333)
  end

  def word(i)
    @words[i][rand(8)]
  end
    
  def test_message_maker
    link = 'http://shortish/'

    # Basically just fuzzing... sending a bunch of random combinations of
    #  weird lengths and see we get nothing back that's longer than 140 bytes
    pt = PDMLTweet.new("a", "b", "c", "d")
    50.times do |t|
      args = ['title', 'author', 'text']
      # Title up to 10 words
      # author up to 2
      # pre up to 50
      rand(10).times { args[0] << " #{word(0)}" }
      rand(2).times { args[1] << " #{word(1)}" }
      rand(50).times { args[2] << " #{word(2)}" }
      m = pt.make_message(args[0], args[1], args[2], link)
      assert(m.bytesize <= 140)
    end
  end

  def test_title_finder
    pt = PDMLTweet.new("a", "b", "c", "d")
    should_find = [ "PESO -- title", "PESO -- title ",
                    "PESO title ", "PESO title",
                    "PESO: title ", "PESO:   title", "PESO : title",
                    "PESO- title", "PESO - title ", "PESO -title "
                  ]
    should_not_find = [ "Peso: whatever", "My great pic", " PEsO : xx",
                        "PESO  \n\r\n"
                      ]
    should_find.each do |s|
      s = "Subject: " + s
      assert((pt.find_title([s]) =~ /^title/), "Sy=[[#{s}]]")
    end
    should_not_find.each do |s|
      s = "Subject: " + s
      assert_equal(nil, pt.find_title([s]), "Sn=[[#{s}]]")
    end
  end

  def test_link_finder
    pt = PDMLTweet.new("a", "b", "c", "d")
    link = 'http://example.com/.-%03fg__+=!!##'
    link2 = 'http://foo.com/bar'
    headers = 'From tbray@chapai.luther.ca  Wed Apr 30 23:55:16 2008
X-Original-To: tbray@chapai.luther.ca
X-Virus-Scanned: Debian amavisd-new at chapai.luther.ca
From: root@chapai.luther.ca (Cron Daemon)
To: tbray@chapai.luther.ca
Subject: Cron <tbray@chapai> sh $HOME/bin/ong
Content-Type: text/plain; charset=UTF-8
X-Cron-Env: <SHELL=/bin/sh>
X-Cron-Env: <HOME=/home/tbray.org/www>
X-Cron-Env: <PATH=/usr/bin:/bin>
X-Cron-Env: <LOGNAME=tbray>
Date: Wed, 30 Apr 2008 23:55:02 -0700 (PDT)'
    
    patterns = [ "#{headers}\r\n\r\n   whatever #{link}",
                 "#{headers}\r\n\r\n#{link}\n\n",
                 "#{headers}\r\n\r\nlinke here: <#{link}> #{link2}\n\n",
                 "#{headers}\r\n\r\n  #{link} >\n\n",
                 "G'day All,

Have you ever gone through a time in your life when you collected
something really, really stupid for no sensible reason?

I have.

Small (500x427 pixels, ~210 kb):
<#{link}>

Large (950x896 pixels, ~350kb):
<http://farm4.static.flickr.com/3084/2461437462_40be4913af_o.jpg>"
               ]
    patterns.each do |pattern|
      p, l = pt.find_link(pattern)
      assert_equal(link, l, "P=#{pattern[headers.length .. -1]}")
    end
  end
end
